#!/usr/bin/env node

require('default-debug')('info')

var scrape = require("scrape-url");
var strip = require("strip");
var io = require("level-json")('./eksi-db');
var parallel = require("parallel-loop");
var debug = require("debug")('eksi');
var info = require("debug")('info');
var log = require('single-line-log')
var styled = require('styled')

var decode = require("ent").decode;
var start = process.argv[2] ? Number(process.argv[2]) : 1;
var end;

function get (id, callback) {
  io.get('sozluk:entry:' + id, function (_, rec) {

    if (rec && rec.id == id) {
      debug('%s is already saved', id)
      return callback()
    }
    
    scrape('https://eksisozluk.com/entry/' + id, ['#title span', '.content', 'footer .entry-date', 'footer address span'], function (error, title, content, time, author) {
      if (error) return callback(error);

      var entry = {
        id: id,
        author: decode(author[0].html()),
        ts: Number(ts(time[0].html())),
        title: decode(title[0].html()),
        content: clean(content[0].html())
      };

      save(entry, callback);
    });
  });
}

function ts (html) {
  return +(new Date(html.match(/datetime=(?:'|")([^"']+)/)[1]));
}

function clean (entry) {
  return strip(markdown(entry.replace(/<br\s*\/?>/g, '\n')));
}

function markdown (entry) {
  return entry.replace(/<a([^>]+)>(.+?)<\/a>/g, function (html, linkInner, caption) {
    var url = linkInner.match(/href\=(?:"|')([^"']+)/)[1];

    if (url.slice(0, 4) == '/?q=') {
      url = '/' + url.slice(4);
    }

    return '['+caption+']('+url+')';
  });
}


function save (entry, callback) {
  var key = 'sozluk:entry:' + entry.id;

  io.set(key, entry, function (error) {
    if (error) throw error;

    key = 'sozluk:title:' + entry.title + ':' + entry.ts;

    io.set(key, entry.id, function (error) {
      if (error) throw error;

      key = 'sozluk:author:' + entry.title + ':' + entry.ts;

      io.set(key, entry.id, function (error) {
        if (error) throw error;

        debug('Saved %s', entry.id);

        callback();
      });
    });
  });
}

function tick (start, callback) {
  var i = start;
  var counter = 0;
  var fail = 0;
  var saved = 0;
  var lastError;
  var lastFailedEntry;

  info('Tick %d - %d', i, i + 9999)
  process.stdout.write('  ');

  parallel(9999, each, function () {
    log.clear()
    callback(i + 9999);
  });

  function each (done, t) {
    var id = i + t;

    get(id, function (error, rec) {
      counter++;

      if (error) {
        fail++;
	console.error('Failed to save %d. Error: %s', id, error.message);
	lastError = error.message;
	lastFailedEntry = id;
      } else {
	saved++;
      }
      
      log('  ' + styled.yellow('%'+Math.floor((counter*100)/9999))+' Current Entry: #' + id + ' Saved: ' + saved + ' Failed: ' + fail + ' Last Fail: #' + (lastFailedEntry || 'n/a') + ', "' + (lastError || 'n/a') + '"')

      done();
    });
  }

}

(function next (i) {

  var start = Date.now();
  var done;

  tick(i, function (cursor) {
    done = true;
    info('Done %dth tick.', cursor);
    next(cursor);
  });

  setTimeout(function () {
    if (done) return;
    info('Timeout at %dth tick. Restarting...', i)
    next(i);
  }, 60 * 60 * 1000);

}(start));
